# -*- coding: utf-8 -*-
"""
    KPortageTray - A set of basic operations for Portage.
    
    Copyright (C) 2010  Ronan Arraes Jardim Chagas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 3 as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """

##\file KPT_Portage.py
##\brief Class to interact with portage.
## File that contains classes to interact with portage and its tools.
##\author Ronan Arraes Jardim Chagas


"""
Imports
"""
import os
import time
from PyQt4 import QtCore
from PyKDE4.kdecore import i18n

# Local imports
from KPT_Definitions import *

class KPT_PortageCheckUpgrades(QtCore.QThread):
	"""
	Thread to verify the number of upgrades that will be done.
	"""

	## Identify if there are a new portage version.
	newPortageVersion = None

	## Number of upgrades returned by the script \c __verifyUpgradeScript.
	numUgrades = None

	## Output of script \c _checkUpgradesScript.
	procesStdout = None

	## Instance of KPT_PortageSync.
	_threadSync = None

	## Path to bash script that check upgrades on the system.
	_script = None

	## KPT_PortageVerifyUpgrades constructor.
	def __init__(self, parent=None):
		QtCore.QThread.__init__(self, parent)
		self._script =  KPORTAGETRAY_INSTALLDIR+"/scripts/checkUpgrades.sh "+os.environ['HOME']+"/.kpt.tmp"

	## KPT_PortageVerifyUpgrades destructor.
	def __del__(self):
		self.wait()

	## Method called when the thread is started.
	def run(self):
		# Wait the sync process to be stopped.
		while(self._threadSync.isRunning()):
			time.sleep(2)
		process = QtCore.QProcess()
		process.start(self._script)
		process.waitForFinished(-1)
		try:
			self.numUpgrades = int(process.readLine())
		except TypeError:
			self.numUpgrade = -1
			self.processStdout = i18n("An unidentified error occured!\nTry to run the process again.")
		else:
			self.processStdout = process.readAll()

		# Verify is there are a new portage version.
		if(QtCore.QString(self.processStdout).contains("sys-apps/portage")):
			self.newPortageVersion = True
		else:
			self.newPortageVersion = False

	#
	# Sets
	#

	## Set the used instance of KPT_PortageSync.
	def setThreadSync(self, threadSync):
		self._threadSync = threadSync

class KPT_PortageDepClean(QtCore.QThread):
	"""
	Thread to execute dep-clean.
	"""

	## Path to bash script that executes dep-clean.
	_script = None

	## KPT_PortageDepClean constructor.
	def __init__(self, parent = None):
		QtCore.QThread.__init__(self, parent)
		self._script =  KPORTAGETRAY_INSTALLDIR+"/scripts/depClean.sh"

	## KPT_PortageEtcUpdate destructor.
	def __del__(self):
		self.wait()

	## Method called when the thread is started.
	def run(self):
		process = QtCore.QProcess()
		process.start(self._script)
		process.waitForFinished(-1)

class KPT_PortageEtcUpdate(QtCore.QThread):
	"""
	Thread to execute etc-update.
	"""

	## Path to bash script that executes etc-update.
	_script = None

	## KPT_PortageEtcUpdate constructor.
	def __init__(self, parent = None):
		QtCore.QThread.__init__(self, parent)
		self._script =  KPORTAGETRAY_INSTALLDIR+"/scripts/etcUpdate.sh"

	## KPT_PortageEtcUpdate destructor.
	def __del__(self):
		self.wait()

	## Method called when the thread is started.
	def run(self):
		process = QtCore.QProcess()
		process.start(self._script)
		process.waitForFinished(-1)

class KPT_PortageRevdepRebuild(QtCore.QThread):
	"""
	Thread to execute revdep-rebuild.
	"""

	## Path to bash script that executes revdep-rebuild.
	_script = None

	## KPT_PortageEtcUpdate constructor.
	def __init__(self, parent = None):
		QtCore.QThread.__init__(self, parent)
		self._script =  KPORTAGETRAY_INSTALLDIR+"/scripts/revdepRebuild.sh"

	## KPT_PortageEtcUpdate destructor.
	def __del__(self):
		self.wait()

	## Method called when the thread is started.
	def run(self):
		process = QtCore.QProcess()
		process.start(self._script)
		process.waitForFinished(-1)

class KPT_PortageSync(QtCore.QThread):
	"""
	Thread to sync portage tree.
	"""

	## Path to bash script that sync the portage tree.
	_script = None

	## KPT_PortageSync constructor.
	def __init__(self, parent = None):
		QtCore.QThread.__init__(self, parent)
		self._script =  KPORTAGETRAY_INSTALLDIR+"/scripts/syncPortageTree.sh"

	## KPT_PortageSync destructor.
	def __del__(self):
		self.wait()

	## Method called when the thread is started.
	def run(self):
		process = QtCore.QProcess()
		process.start(self._script)
		process.waitForFinished(-1)

class KPT_PortageUpgrade(QtCore.QThread):
	"""
	Thread to upgrade world set.
	"""

	## Path to bash script that upgrade the system.
	_script = None

	## KPT_PortageUpgrade constructor.
	def __init__(self, parent = None):
		QtCore.QThread.__init__(self, parent)
		self._script = KPORTAGETRAY_INSTALLDIR+"/scripts/upgrade.sh"

	## KPT_PortageUpgrade destructor.
	def __del__(self):
		self.wait()

	## Method called when the thread is started.
	def run(self):
		process = QtCore.QProcess()
		process.start(self._script)
		process.waitForFinished(-1)

class KPT_PortageUpgradePortage(QtCore.QThread):
	"""
	Thread to upgrade portage.
	"""

	## Path to bash script that upgrade portage.
	_script = None

	## KPT_PortageUpgradePortage constructor.
	def __init__(self, parent = None):
		QtCore.QThread.__init__(self, parent)
		self._script = KPORTAGETRAY_INSTALLDIR+"/scripts/upgradePortage.sh"

	## KPT_PortageUpgrade destructor.
	def __del__(self):
		self.wait()

	## Method called when the thread is started.
	def run(self):
		process = QtCore.QProcess()
		process.start(self._script)
		process.waitForFinished(-1)

class KPT_Portage:
	"""
	Class to interact with portage.
	"""

	## KPT_Portage constructor.
	def __init__(self):
		pass
	#
	# Funcions
	#

	## Returns the text generated by genlop -c --nocolor
	def genlopText(self):
		genlopProcess = QtCore.QProcess()
		genlopProcess.start('genlop -c --nocolor')
		genlopProcess.waitForFinished(-1)
		genlopText = genlopProcess.readAll()

		if 'Error' in genlopText.split('\n')[0]:
			genlopText = i18n("No packages are being compiled or occured an error with genlop.")

		return genlopText
